@@ -1,21 +1,21 @@
 import unittest
 
-from uritools import uricompose
+from uritools import uricompose, urisplit
 
 
 class UriComposeTest(unittest.TestCase):
 
     def check(self, expected, **kwargs):
-        result = uricompose(**kwargs)
+        uri = uricompose(**kwargs)
         self.assertEqual(
-            result, expected,
-            '%r -> %r != %r' % (kwargs, result, expected)
+            expected, uri,
+            '%r -> %r != %r' % (kwargs, uri, expected)
         )
 
     def test_rfc3986(self):
         """uricompose test cases from [RFC3986] 3. Syntax Components"""
         self.check(
-            'foo://example.com:8042/over/there?name=ferret#nose',
+            b'foo://example.com:8042/over/there?name=ferret#nose',
             scheme='foo',
             authority='example.com:8042',
             path='/over/there',
@@ -23,15 +23,14 @@ class UriComposeTest(unittest.TestCase):
             fragment='nose'
         )
         self.check(
-            'urn:example:animal:ferret:nose',
+            b'urn:example:animal:ferret:nose',
             scheme='urn',
             path='example:animal:ferret:nose'
         )
 
     def test_querylist(self):
-        # FIXME: empty query?
         self.check(
-            'foo://example.com:8042/over/there?#nose',
+            b'foo://example.com:8042/over/there?#nose',
             scheme='foo',
             authority='example.com:8042',
             path='/over/there',
@@ -39,7 +38,7 @@ class UriComposeTest(unittest.TestCase):
             fragment='nose'
         )
         self.check(
-            'foo://example.com:8042/over/there?name=ferret#nose',
+            b'foo://example.com:8042/over/there?name=ferret#nose',
             scheme='foo',
             authority='example.com:8042',
             path='/over/there',
@@ -47,15 +46,23 @@ class UriComposeTest(unittest.TestCase):
             fragment='nose'
         )
         self.check(
-            'foo://example.com:8042/over/there?name=ferret#nose',
+            b'foo://example.com:8042/over/there?id=42#nose',
             scheme='foo',
             authority='example.com:8042',
             path='/over/there',
-            query=['name=ferret'],
+            query=[('id', 42)],
             fragment='nose'
         )
         self.check(
-            'foo://example.com:8042/over/there?name=swallow&type=African#beak',
+            b'foo://example.com:8042/over/there?none#nose',
+            scheme='foo',
+            authority='example.com:8042',
+            path='/over/there',
+            query=[('none', None)],
+            fragment='nose'
+        )
+        self.check(
+            b'foo://example.com:8042/over/there?name=swallow&type=African#beak',  # noqa
             scheme='foo',
             authority='example.com:8042',
             path='/over/there',
@@ -64,9 +71,8 @@ class UriComposeTest(unittest.TestCase):
         )
 
     def test_querydict(self):
-        # FIXME: empty query?
         self.check(
-            'foo://example.com:8042/over/there?#nose',
+            b'foo://example.com:8042/over/there?#nose',
             scheme='foo',
             authority='example.com:8042',
             path='/over/there',
@@ -74,10 +80,46 @@ class UriComposeTest(unittest.TestCase):
             fragment='nose'
         )
         self.check(
-            'foo://example.com:8042/over/there?name=ferret#nose',
+            b'foo://example.com:8042/over/there?name=ferret#nose',
             scheme='foo',
             authority='example.com:8042',
             path='/over/there',
             query={'name': 'ferret'},
             fragment='nose'
         )
+        self.check(
+            b'foo://example.com:8042/over/there?id=42#nose',
+            scheme='foo',
+            authority='example.com:8042',
+            path='/over/there',
+            query={'id': 42},
+            fragment='nose'
+        )
+        self.check(
+            b'foo://example.com:8042/over/there?none#nose',
+            scheme='foo',
+            authority='example.com:8042',
+            path='/over/there',
+            query={'none': None},
+            fragment='nose'
+        )
+        self.assertEqual(
+            {'name': ['swallow'], 'type': ['African']},
+            urisplit(uricompose(
+                scheme='foo',
+                authority='example.com:8042',
+                path='/over/there',
+                query={'name': 'swallow', 'type': 'African'},
+                fragment='beak'
+            )).getquerydict()
+        )
+        self.assertEqual(
+            {'name': ['swallow'], 'type': ['African']},
+            urisplit(uricompose(
+                scheme='foo',
+                authority='example.com:8042',
+                path='/over/there',
+                query={'name': ['swallow'], 'type': ['African']},
+                fragment='beak'
+            )).getquerydict()
+        )
