@@ -1,15 +1,42 @@
-from .const import SUB_DELIMS
-from .encoding import uriencode
-from .regex import _SCHEME_RE, _AUTHORITY_RE, _IPV6_ADDRESS_RE
-from .split import uriunsplit
+from __future__ import unicode_literals
+
+import re
 
 from collections import Iterable, Mapping
 
 try:
-    String = basestring
+    String = (basestring, bytearray)
 except NameError:
     String = (str, bytes, bytearray)
 
+from .const import SUB_DELIMS
+from .parse import uriunsplit
+from .encoding import uriencode
+from .ipaddress import ip_address
+
+# RFC 3986 3.1: scheme = ALPHA *( ALPHA / DIGIT / "+" / "-" / "." )
+SCHEME_RE = re.compile(br"\A[A-Za-z][A-Za-z0-9+.-]*\Z")
+
+# RFC 3986 3.2: authority = [ userinfo "@" ] host [ ":" port ]
+AUTHORITY_RE = re.compile(r"\A(?:(.*)@)?(.*?)(?::([0-9]*))?\Z")
+
+
+def decoder(encoding):
+    def decode(obj):
+        return obj.decode(encoding) if hasattr(obj, 'decode') else obj
+    return decode
+
+
+def splitauth(authority, encoding):
+    if isinstance(authority, type('')):
+        return AUTHORITY_RE.match(authority).groups()
+    elif isinstance(authority, String):
+        return AUTHORITY_RE.match(authority.decode(encoding)).groups()
+    elif isinstance(authority, Iterable):
+        return map(decoder(encoding), authority)
+    else:
+        raise TypeError('Invalid authority type')
+
 
 def querylist(items, delim, encoding):
     safe = (SUB_DELIMS + b':@/?').replace(delim, b'')
@@ -58,15 +85,17 @@ def uricompose(scheme=None, authority=None, path='', query=None,
     # "HTTP" as well as "http") for the sake of robustness but should
     # only produce lowercase scheme names for consistency.
     if scheme is not None:
-        if not _SCHEME_RE.match(scheme):
-            raise ValueError('Invalid scheme: %r' % scheme)
-        scheme = scheme.lower().encode()
+        if isinstance(scheme, type('')):
+            encoded_scheme = scheme.encode()
+        else:
+            encoded_scheme = scheme
+        if SCHEME_RE.match(encoded_scheme):
+            scheme = encoded_scheme.lower()
+        else:
+            raise ValueError('Invalid scheme %r' % scheme)
 
     if authority is not None:
-        if isinstance(authority, String):
-            userinfo, host, port = _AUTHORITY_RE.match(authority).groups()
-        else:
-            userinfo, host, port = authority
+        userinfo, host, port = splitauth(authority, encoding)
         # RFC 3986 3.2.1: The user information, if present, is
         # followed by a commercial at-sign ("@") that delimits it from
         # the host.
@@ -79,23 +108,25 @@ def uricompose(scheme=None, authority=None, path='', query=None,
         # and hexadecimal addresses for the sake of uniformity, while
         # only using uppercase letters for percent-encodings.
         if host is None:
-            raise ValueError('Host subcomponent must be present if empty')
-        host = host.lower()
-        if host.startswith('[') and host.endswith(']'):
-            authority += uriencode(host, SUB_DELIMS + b':')  # TODO: check IPv6
-        elif _IPV6_ADDRESS_RE.match(host):
-            authority += b'[' + uriencode(host, SUB_DELIMS + b':') + b']'
+            raise ValueError('URI host component must be present if empty')
+        elif host.startswith('[') and host.endswith(']'):
+            authority += b'[' + ip_address(host[1:-1]).encode() + b']'
+        elif host.startswith('[') or host.endswith(']'):
+            raise ValueError('Invalid host %r' % host)
         else:
-            authority += uriencode(host, SUB_DELIMS, encoding)
+            try:
+                authority += b'[' + ip_address(host).encode() + b']'
+            except ValueError:
+                authority += uriencode(host, SUB_DELIMS, encoding).lower()
         # RFC 3986 3.2.3: URI producers and normalizers should omit
         # the port component and its ":" delimiter if port is empty or
         # if its value would be the same as that of the scheme's
         # default.
-        port = str(port).encode(encoding) if port is not None else b''
+        port = str(port).encode() if port is not None else b''
         if port.isdigit():
             authority += b':' + port
         elif port:
-            raise ValueError('Non-decimal port: %s' % port)
+            raise ValueError('Invalid port %r' % port)
 
     # RFC 3986 3.3: If a URI contains an authority component, then the
     # path component must either be empty or begin with a slash ("/")
@@ -106,23 +137,26 @@ def uricompose(scheme=None, authority=None, path='', query=None,
     # character.
     if path is None:
         raise ValueError('URI path component must be present if empty')
-    if authority is not None and path and not path.startswith('/'):
+    else:
+        path = uriencode(path, SUB_DELIMS + b':@/', encoding)
+    if authority is not None and path and not path.startswith(b'/'):
         raise ValueError('Invalid path %r with authority component' % path)
-    if authority is None and path.startswith('//'):
+    if authority is None and path.startswith(b'//'):
         raise ValueError('Invalid path %r without authority component' % path)
-    if scheme is None and authority is None and ':' in path.split('/', 1)[0]:
-        raise ValueError('Invalid path %r without scheme component' % path)
-    path = uriencode(path, SUB_DELIMS + b':@/', encoding)
+    if scheme is None and authority is None and path.find(b':') < path.find(b'/'):  # noqa
+        raise ValueError('Invalid relative-path reference %r' % path)
 
     if query is not None:
         if isinstance(query, String):
             query = uriencode(query, SUB_DELIMS + b':@/?', encoding)
         elif isinstance(query, Mapping):
             query = querydict(query, delim, encoding)
-        else:
+        elif isinstance(query, Iterable):
             query = querylist(query, delim, encoding)
+        else:
+            raise TypeError('Invalid query type')
 
-    if fragment:
+    if fragment is not None:
         fragment = uriencode(fragment, SUB_DELIMS + b':@/?', encoding)
 
     return uriunsplit((scheme, authority, path, query, fragment))
